@@ -30,11 +30,22 @@ register_s3_method <- function(pkg, generic, class, fun = NULL) {
 
 detect_multicore_support = function() {
   ns = getNamespace("parallel")
-  supportedByOS = exists("mcparallel", mode = "function",
-                         envir = ns, inherits = FALSE)
+  supportedByOS = exists(
+    "mcparallel",
+    mode = "function",
+    envir = ns,
+    inherits = FALSE
+  )
   return(supportedByOS)
 }
 
 skpr_system_setup_env = new.env(parent = emptyenv())
 
-assign("has_multicore_support", detect_multicore_support(), envir = skpr_system_setup_env)
+assign(
+  "has_multicore_support",
+  detect_multicore_support(),
+  envir = skpr_system_setup_env
+)
+
+skpr_moment_matrix_cache = new.env(parent = emptyenv())
+
